package com.zettenet.recipesdb.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;
import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;
import org.hibernate.annotations.Sort;
import org.hibernate.annotations.SortType;

import com.zettenet.recipesdb.web.models.INamedComparablePersistentObject;

/**
 * A recipe composed of a title, a source, a list of ingredients, a method of preparation, a
 * description of the serving size, and a list of tags.
 * 
 * @author zetten
 */
@Entity
@Table(name = "recipe")
public class Recipe implements INamedComparablePersistentObject<Recipe>, Serializable {
	private static final long serialVersionUID = -6801505291523863587L;

	/**
	 * The set of categories in which the recipe has been placed.
	 */
	@OneToMany(mappedBy = "recipe", orphanRemoval = true, fetch = FetchType.EAGER)
	@Fetch(FetchMode.SUBSELECT)
	@Cascade(CascadeType.ALL)
	private Set<RecipeCategoryEntry> categories = new HashSet<RecipeCategoryEntry>();

	/**
	 * The unique identifier of the recipe.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	/**
	 * The ordered list of ingredients.
	 */
	@OneToMany(mappedBy = "recipe", orphanRemoval = true, fetch = FetchType.EAGER)
	@Sort(type = SortType.NATURAL)
	@Fetch(FetchMode.SUBSELECT)
	@Cascade(CascadeType.ALL)
	private List<RecipeIngredientEntry> ingredients = new ArrayList<RecipeIngredientEntry>();

	/**
	 * The method of preparation.
	 */
	@OneToOne(mappedBy = "recipe", orphanRemoval = true)
	@Fetch(FetchMode.JOIN)
	@Cascade(CascadeType.ALL)
	private RecipeMethod method;

	/**
	 * The size or number of servings.
	 */
	private String servings;

	/**
	 * The source.
	 */
	@ManyToOne
	@PrimaryKeyJoinColumn
	@Fetch(FetchMode.JOIN)
	private RecipeSource source;

	/**
	 * The name of the recipe.
	 */
	@NotNull
	private String name;

	/**
	 * Constructor.
	 */
	public Recipe() {
	}

	/**
	 * Adds the specified category to the set of categories this recipe has been tagged with. Also
	 * adds the recipe to the category's list of associated recipes.
	 * 
	 * @param category
	 *            the category to add
	 * @return true if the set of categories did not already contain the specified category
	 */
	public boolean addCategory(RecipeCategory category) {
		boolean hasCat = false;
		
		for (RecipeCategoryEntry catEntry : this.categories) {
			if (catEntry.getCategory().getName().equals(category.getName())) {
				hasCat = true;
				break;
			}
		}
		
		if (!hasCat) {
			categories.add(new RecipeCategoryEntry(this, category));
			return true;
		} else {
			return false;
		}
	}

	public boolean removeCategory(RecipeCategory category) {
		RecipeCategoryEntry catToRemove = null;
		
		for (RecipeCategoryEntry catEntry : this.categories) {
			if (catEntry.getCategory().getName().equals(category.getName())) {
				catToRemove = catEntry;
				break;
			}
		}
		
		if (catToRemove != null) {
			categories.remove(catToRemove);
			return true;
		} else {
			return false;
		}
	}

	@Override
	public int compareTo(Recipe that) {
		if (that == null) {
			return -1;
		}
		
		if (this == that) {
			return 0;
		}

		return name.compareTo(that.name);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		Recipe other = (Recipe) obj;
		if (categories == null) {
			if (other.categories != null) {
				return false;
			}
		} else if (!isCategoriesSetsEqual(categories, other.categories)) {
			return false;
		}
		if (ingredients == null) {
			if (other.ingredients != null) {
				return false;
			}
		} else if (!isIngredientsListsEqual(ingredients, other.ingredients)) {
			return false;
		}
		if (method == null) {
			if (other.method != null) {
				return false;
			}
		} else if (!method.equals(other.method)) {
			return false;
		}
		if (servings == null) {
			if (other.servings != null) {
				return false;
			}
		} else if (!servings.equals(other.servings)) {
			return false;
		}
		if (source == null) {
			if (other.source != null) {
				return false;
			}
		} else if (!source.equals(other.source)) {
			return false;
		}
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		return true;
	}

	private boolean isCategoriesSetsEqual(
			Set<RecipeCategoryEntry> leftCategories,
			Set<RecipeCategoryEntry> rightCategories) {
		boolean equal = true;
		
		for (RecipeCategoryEntry leftEntry : leftCategories) {
			equal = rightCategories.contains(leftEntry);
		}
		
		return equal;
	}

	/**
	 * This is necessary to get around Hibernate's insistence on wrapping Lists
	 * in a Bag and then not implementing a proper .equals() method. 
	 * @param leftIngredients
	 * @param rightIngredients
	 * @return
	 */
	private boolean isIngredientsListsEqual(List<RecipeIngredientEntry> leftIngredients, List<RecipeIngredientEntry> rightIngredients) {
		boolean equal = true;
		
		for (int i = 0; i < leftIngredients.size(); i++) {
			equal = leftIngredients.get(i).equals(rightIngredients.get(i));
		}
		
		return equal;
	}

	/**
	 * @return the set of categories in which the recipe has been placed
	 */
	public Set<RecipeCategoryEntry> getCategories() {
		return categories;
	}

	/**
	 * Gets the unique identifier of the recipe.
	 * 
	 * @return long version of the serializable id stored in the database
	 */
	public Long getId() {
		return id;
	}

	/**
	 * @return the ordered list of ingredients
	 */
	public List<RecipeIngredientEntry> getIngredients() {
		return ingredients;
	}

	/**
	 * @return the method of preparation
	 */
	public RecipeMethod getMethod() {
		return method;
	}

	/**
	 * @return the size or number of servings
	 */
	public String getServings() {
		return servings;
	}

	/**
	 * @return the source
	 */
	public RecipeSource getSource() {
		return source;
	}

	/**
	 * @return the name of the recipe
	 */
	public String getName() {
		return name;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((categories == null) ? 0 : categories.hashCode());
		result = prime * result + ((ingredients == null) ? 0 : ingredients.hashCode());
		result = prime * result + ((method == null) ? 0 : method.hashCode());
		result = prime * result + ((servings == null) ? 0 : servings.hashCode());
		result = prime * result + ((source == null) ? 0 : source.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	/**
	 * @param categories
	 *            the set of categories in which the recipe should be placed
	 */
	public void setCategories(Set<RecipeCategory> categories) {
		this.categories = new HashSet<RecipeCategoryEntry>();

		if (categories != null) {
			for (RecipeCategory cat : categories) {
				if (cat != null) {
					this.categories.add(new RecipeCategoryEntry(this, cat));
				}
			}
		}
	}

	/**
	 * Sets the unique identifier of this object. Should not be used manually.
	 */
	public void setId(Long id) {
		this.id = id;
	}

	/**
	 * @param ingredients
	 *            the ordered list of ingredients
	 */
	public void setIngredients(List<RecipeIngredientEntry> ingredients) {
		this.ingredients = ingredients;

		if (this.ingredients != null) {
			for (RecipeIngredientEntry ingr : this.ingredients) {
				if (ingr != null) {
					ingr.setRecipe(this);
				}
			}
		}
	}

	/**
	 * @param method
	 *            the method of preparation
	 */
	public void setMethod(RecipeMethod method) {
		this.method = method;
		if (this.method != null) {
			this.method.setRecipe(this);
		}
	}

	/**
	 * @param servings
	 *            the size or number of servings
	 */
	public void setServings(String servings) {
		this.servings = servings;
	}

	/**
	 * @param source
	 *            the source of the recipe
	 */
	public void setSource(RecipeSource source) {
		this.source = source;
		if (this.source != null) {
			this.source.addRecipe(this);
		}
	}

	/**
	 * @param title
	 *            the name of the recipe
	 */
	public void setName(String name) {
		this.name = name;
	}

	@Override
	public String toString() {
		String ret = "[";

		ret += id + ", ";
		ret += "\"" + name + "\", ";
		ret += source + ", ";
		ret += categories + ", ";
		ret += "\"" + servings + "\", ";
		ret += ingredients + ", ";
		ret += method + "]";

		return ret;
	}
	
}
