package com.zettenet.recipesdb.model;

import java.io.Serializable;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

/**
 * A link between a Recipe and a RecipeIngredient, containing the amount and the
 * RecipeIngredientGroup to which the entry belongs.
 * 
 * @author pvzetten
 */
@Entity
@Table(name = "recipe_ingredient_link", uniqueConstraints = { @UniqueConstraint(columnNames = {
		"recipe_id", "sequence" }) })
public class RecipeIngredientEntry implements Comparable<RecipeIngredientEntry>, Serializable {
	private static final long serialVersionUID = -8449967585024484210L;

	/**
	 * The amount of the ingredient.
	 */
	private String amount;

	/**
	 * The group to which the entry belongs.
	 */
	@ManyToOne
	@PrimaryKeyJoinColumn
	private RecipeIngredientGroup group;

	/**
	 * The unique identifier of the entry.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	/**
	 * The ingredient.
	 */
	@ManyToOne
	@PrimaryKeyJoinColumn
	private RecipeIngredient ingredient;

	/**
	 * The recipe to which the entry belongs.
	 */
	@ManyToOne
	@PrimaryKeyJoinColumn(name = "recipe_id")
	private Recipe recipe;

	/**
	 * The position of the entry in the ordered List of ingredients.
	 */
	private int sequence;

	/**
	 * Constructor.
	 */
	public RecipeIngredientEntry() {
	}

	/**
	 * Constructor.
	 * 
	 * @param sequence
	 *            the position of the entry in the ordered list of ingredients
	 * @param amount
	 *            the amount of the ingredient
	 * @param ingredient
	 *            the ingredient
	 * @param group
	 *            the group to which the entry belongs
	 */
	public RecipeIngredientEntry(int sequence, String amount, RecipeIngredient ingredient,
			RecipeIngredientGroup group) {
		this.sequence = sequence;
		this.amount = amount;
		this.ingredient = ingredient;
		this.group = group;
	}

	@Override
	public int compareTo(RecipeIngredientEntry that) {
		if (that == null) {
			return -1;
		}

		if (this == that) {
			return 0;
		}

		if (sequence < that.sequence) {
			return -1;
		} else if (sequence == that.sequence) {
			return 0;
		} else {
			return 1;
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}

		if (obj == null) {
			return false;
		}

		if (getClass() != obj.getClass()) {
			return false;
		}

		RecipeIngredientEntry other = (RecipeIngredientEntry) obj;

		if (amount == null) {
			if (other.amount != null) {
				return false;
			}
		} else if (!amount.equals(other.amount)) {
			return false;
		}

		if (group == null) {
			if (other.group != null) {
				return false;
			}
		} else if (!group.equals(other.group)) {
			return false;
		}

		if (ingredient == null) {
			if (other.ingredient != null) {
				return false;
			}
		} else if (!ingredient.equals(other.ingredient)) {
			return false;
		}

		if (recipe == null) {
			if (recipe.getId() != other.recipe.getId()) {
				return false;
			}
		}

		if (sequence != other.sequence) {
			return false;
		}

		return true;
	}

	/**
	 * @return the amount of the ingredient
	 */
	public String getAmount() {
		return amount;
	}

	/**
	 * @return the group to which the entry belongs
	 */
	public RecipeIngredientGroup getGroup() {
		return group;
	}

	/**
	 * Gets the unique identifier of the entry.
	 * 
	 * @return long version of the serializable id stored in the database.
	 */
	public Long getId() {
		return id;
	}

	/**
	 * @return the ingredient
	 */
	public RecipeIngredient getIngredient() {
		return ingredient;
	}

	/**
	 * @return the recipe to which the entry belongs
	 */
	public Recipe getRecipe() {
		return recipe;
	}

	/**
	 * @return the position of the entry in the ordered List of ingredients
	 */
	public int getSequence() {
		return sequence;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((amount == null) ? 0 : amount.hashCode());
		result = prime * result + ((group == null) ? 0 : group.hashCode());
		result = prime * result + ((ingredient == null) ? 0 : ingredient.hashCode());
		result = (int) (prime * result + ((recipe == null) ? 0 : (recipe.getId() == null ? 0 : recipe.getId())));
		result = prime * result + sequence;
		return result;
	}

	/**
	 * @param amount
	 *            the amount of the ingredient
	 */
	public void setAmount(String amount) {
		this.amount = amount;
	}

	/**
	 * @param group
	 *            the group to which the entry belongs
	 */
	public void setGroup(RecipeIngredientGroup group) {
		this.group = group;
	}

	/**
	 * Sets the unique identifier of this object. Should not be used manually.
	 */
	@SuppressWarnings("unused")
	private void setId(Long id) {
		this.id = id;
	}

	/**
	 * @param ingredient
	 *            the ingredient
	 */
	public void setIngredient(RecipeIngredient ingredient) {
		this.ingredient = ingredient;
	}

	/**
	 * @param recipe
	 *            the recipe to which this entry belongs
	 */
	public void setRecipe(Recipe recipe) {
		this.recipe = recipe;
	}

	/**
	 * @param sequence
	 *            the position of the entry in the ordered List of ingredients
	 */
	public void setSequence(int sequence) {
		this.sequence = sequence;
	}

	@Override
	public String toString() {
		String ret = "[";

		ret += id + ", ";
		ret += recipe.getId() + ", ";
		ret += sequence + ", ";
		ret += "\"" + amount + "\", ";
		ret += ingredient + ", ";
		ret += group + "]";

		return ret;
	}

}
