package com.zettenet.recipesdb.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;
import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Parameter;
import org.hibernate.annotations.Sort;
import org.hibernate.annotations.SortType;

/**
 * The method of preparation attached to a specific recipe.
 * 
 * @author pvzetten
 */
@Entity
@Table(name = "recipe_method")
public class RecipeMethod implements Serializable {
	private static final long serialVersionUID = 2793300428107778900L;

	/**
	 * The unique identifier of the method.
	 */
	@Id
	@GenericGenerator(name = "genId", strategy = "foreign", parameters = { @Parameter(name = "property", value = "recipe") })
	@GeneratedValue(generator = "genId")
	private Long id;

	/**
	 * Whether the list of steps should be displayed with numbers or bullet points.
	 */
	private boolean numberedSteps;

	/**
	 * A note to be displayed before the list of steps.
	 */
	private String postNote;

	/**
	 * A note to be displayed after the list of steps.
	 */
	private String preNote;

	/**
	 * The recipe with which the method is associated.
	 */
	@OneToOne
	@PrimaryKeyJoinColumn
	private Recipe recipe;

	/**
	 * The ordered list of preparation steps
	 */
	@OneToMany(mappedBy = "method", orphanRemoval = true, fetch = FetchType.EAGER)
	@Fetch(FetchMode.SUBSELECT)
	@Sort(type = SortType.NATURAL)
	@Cascade(CascadeType.ALL)
	private List<RecipeStep> steps = new ArrayList<RecipeStep>();

	/**
	 * Constructor.
	 */
	public RecipeMethod() {
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		RecipeMethod other = (RecipeMethod) obj;
		if (numberedSteps != other.numberedSteps) {
			return false;
		}
		if (postNote == null) {
			if (other.postNote != null) {
				return false;
			}
		} else if (!postNote.equals(other.postNote)) {
			return false;
		}
		if (preNote == null) {
			if (other.preNote != null) {
				return false;
			}
		} else if (!preNote.equals(other.preNote)) {
			return false;
		}
		if (recipe == null) {
			if (recipe.getId() != other.recipe.getId()) {
				return false;
			}
		}
		if (steps == null) {
			if (other.steps != null) {
				return false;
			}
		} else if (!isStepsEqual(steps, other.steps)) {
			return false;
		}
		return true;
	}

	/**
	 * Gets the unique identifier of this step.
	 * 
	 * @return long version of the serializable id stored in the database.
	 */
	public Long getId() {
		return id;
	}

	/**
	 * @return a note to be shown after the steps
	 */
	public String getPostNote() {
		return postNote;
	}

	/**
	 * @return a note to be shown before the steps
	 */
	public String getPreNote() {
		return preNote;
	}

	/**
	 * @return the recipe with which the method is associated
	 */
	public Recipe getRecipe() {
		return recipe;
	}

	/**
	 * @return the ordered list of preparation steps
	 */
	public List<RecipeStep> getSteps() {
		return steps;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (numberedSteps ? 1231 : 1237);
		result = prime * result + ((postNote == null) ? 0 : postNote.hashCode());
		result = prime * result + ((preNote == null) ? 0 : preNote.hashCode());
		result = (int) (prime * result + ((recipe == null) ? 0 : (recipe.getId() == null ? 0 : recipe.getId())));
		result = prime * result + ((steps == null) ? 0 : steps.hashCode());
		return result;
	}

	/**
	 * @return whether the list of steps should be displayed with numbers or bullet points
	 */
	public boolean isNumberedSteps() {
		return numberedSteps;
	}

	/**
	 * Sets the unique identifier of this object. Should not be used manually.
	 */
	public void setId(Long id) {
		this.id = id;
		
		for (RecipeStep step : this.steps) {
			step.setId(this.id);
		}
	}

	/**
	 * @param numberedSteps
	 *            Whether the method should display numbered steps.
	 */
	public void setNumberedSteps(boolean numberedSteps) {
		this.numberedSteps = numberedSteps;
	}

	/**
	 * @param postNote
	 *            A note to be shown after the steps.
	 */
	public void setPostNote(String postNote) {
		this.postNote = postNote;
	}

	/**
	 * @param preNote
	 *            A note to be shown before the steps.
	 */
	public void setPreNote(String preNote) {
		this.preNote = preNote;
	}

	/**
	 * @param recipe
	 *            The recipe with which the method is associated.
	 */
	public void setRecipe(Recipe recipe) {
		this.recipe = recipe;
	}

	/**
	 * @param steps
	 *            The ordered list of preparation steps.
	 */
	public void setSteps(List<RecipeStep> steps) {
		this.steps = steps;

		for (RecipeStep step : this.steps) {
			step.setMethod(this);
		}
	}

	@Override
	public String toString() {
		String ret = "[";

		ret += id + ", ";
		ret += numberedSteps + ", ";
		if (preNote != null) {
			ret += "\"" + preNote + "\", ";
		}
		ret += steps + ", ";
		if (postNote != null) {
			ret += "\"" + postNote + "\"";
		}
		ret += "]";

		return ret;
	}

	/**
	 * This is necessary to get around Hibernate's insistence on wrapping Lists
	 * in a Bag and then not implementing a proper .equals() method.
	 * @param leftSteps
	 * @param rightSteps
	 * @return
	 */
	private boolean isStepsEqual(List<RecipeStep> leftSteps, List<RecipeStep> rightSteps) {
		boolean equal = true;
		
		for (int i = 0; i < leftSteps.size(); i++) {
			equal = leftSteps.get(i).equals(rightSteps.get(i));
		}
		
		return equal;
	}
}
