package com.zettenet.recipesdb.model;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import com.zettenet.recipesdb.web.models.INamedComparablePersistentObject;

/**
 * A source of recipes; e.g., "X Magazine", or "Y Website".
 * 
 * @author pvzetten
 */
@Entity
@Table(name = "recipe_source")
public class RecipeSource implements Serializable, INamedComparablePersistentObject<RecipeSource> {
	private static final long serialVersionUID = 7369069817900222639L;

	/**
	 * The unique identifier of the source.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	/**
	 * The set of recipes from the source.
	 */
	@OneToMany(mappedBy = "source")
	private Set<Recipe> recipes = new HashSet<Recipe>();

	/**
	 * The name of the source.
	 */
	@Column(unique = true)
	private String name;

	/**
	 * Constructor.
	 */
	public RecipeSource() {
	}

	/**
	 * Constructor.
	 * 
	 * @param source
	 *            The name of the source.
	 */
	public RecipeSource(String name) {
		this.name = name;
	}

	/**
	 * Adds the specified recipe to the set of recipes from the source.
	 * 
	 * @param recipe
	 *            the recipe to add
	 * @return true if the set of recipes did not already contain the specified recipe
	 */
	public boolean addRecipe(Recipe recipe) {
		return recipes.add(recipe);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		RecipeSource other = (RecipeSource) obj;
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		return true;
	}

	/**
	 * Gets the unique identifier of the source.
	 * 
	 * @return long version of the serializable id stored in the database
	 */
	public Long getId() {
		return id;
	}

	/**
	 * @return the set of recipes from the source
	 */
	public Set<Recipe> getRecipes() {
		return recipes;
	}

	/**
	 * @return the name of the source
	 */
	public String getName() {
		return name;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	/**
	 * Removes the specified recipe from the set of recipes from the source.
	 * 
	 * @param recipe
	 *            the recipe to remove
	 * @return true if the set of recipes contained the specified recipe
	 */
	public boolean removeRecipe(Recipe recipe) {
		return recipes.remove(recipe);
	}

	/**
	 * Sets the unique identifier of this object. Should not be used manually.
	 */
	@SuppressWarnings("unused")
	private void setId(Long id) {
		this.id = id;
	}

	/**
	 * @param recipes
	 *            the set of recipes from the source
	 */
	public void setRecipes(Set<Recipe> recipes) {
		this.recipes = recipes;
	}

	/**
	 * @param source
	 *            the name of the source
	 */
	public void setName(String name) {
		this.name = name;
	}

	@Override
	public String toString() {
		String ret = "[";

		ret += id + ", ";
		ret += "\"" + name + "\"]";

		return ret;
	}

	@Override
	public int compareTo(RecipeSource that) {
		if (that == null) {
			return -1;
		}

		return name.compareTo(that.name);
	}
}
