package com.zettenet.recipesdb.model;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import com.zettenet.recipesdb.web.models.IValuedOrderedComparablePersistentObject;

/**
 * A step in the preparation of a given recipe.
 * 
 * @author pvzetten
 */
/**
 * @author pvzetten
 */
@Entity
@Table(name = "recipe_steps", uniqueConstraints = { @UniqueConstraint(columnNames = { "method_id",
		"sequence" }) })
public class RecipeStep implements IValuedOrderedComparablePersistentObject<RecipeStep>, Serializable {
	private static final long serialVersionUID = 9129915988288760901L;

	/**
	 * The unique identifier of this step.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	/**
	 * The recipe to which this step belongs.
	 */
	@ManyToOne
	@PrimaryKeyJoinColumn(name = "method_id")
	private RecipeMethod method;

	/**
	 * The position of this step in the ordered List of preparation steps.
	 */
	private int sequence;

	/**
	 * The contents of the step.
	 */
	@Column(length = 1024)
	private String step;

	/**
	 * Constructor.
	 */
	public RecipeStep() {
	}

	/**
	 * Constructor.
	 * 
	 * @param sequence
	 *            The position of this step in the ordered List of preparation steps.
	 * @param step
	 *            The contents of the step.
	 */
	public RecipeStep(int sequence, String step) {
		this.sequence = sequence;
		this.step = step;
	}

	@Override
	public int compareTo(RecipeStep that) {
		if (that == null) {
			return -1;
		}

		if (this == that) {
			return 0;
		}

		if (sequence < that.sequence) {
			return -1;
		} else if (sequence == that.sequence) {
			return 0;
		} else {
			return 1;
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}

		if (obj == null) {
			return false;
		}

		if (getClass() != obj.getClass()) {
			return false;
		}

		RecipeStep other = (RecipeStep) obj;

		if (sequence != other.sequence) {
			return false;
		}

		if (step == null) {
			if (other.step != null) {
				return false;
			}
		} else if (!step.equals(other.step)) {
			return false;
		}

		return true;
	}

	/**
	 * Gets the unique identifier of this step.
	 * 
	 * @return long version of the serializable id stored in the database
	 */
	public Long getId() {
		return id;
	}

	/**
	 * @return the recipe to which this step belongs.
	 */
	public RecipeMethod getMethod() {
		return method;
	}

	/**
	 * @return the position of this step in the ordered List of preparation steps
	 */
	public int getSequence() {
		return sequence;
	}

	/**
	 * @return the contents of the step
	 */
	public String getStep() {
		return step;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (int) (prime * result + ((method == null) ? 0 : (method.getId() == null ? 0 : method.getId())));
		result = prime * result + sequence;
		result = prime * result + ((step == null) ? 0 : step.hashCode());
		return result;
	}

	/**
	 * Sets the unique identifier of this object. Should not be used manually.
	 */
	public void setId(Long id) {
		this.id = id;
	}

	/**
	 * @param recipe
	 *            the recipe to which this recipe belongs
	 */
	public void setMethod(RecipeMethod method) {
		this.method = method;
	}

	/**
	 * @param sequence
	 *            the position of this step in the ordered List of preparation steps
	 */
	public void setSequence(int sequence) {
		this.sequence = sequence;
	}

	/**
	 * @param step
	 *            the contents of the step
	 */
	public void setStep(String step) {
		this.step = step;
	}

	@Override
	public String toString() {
		String ret = "[";

		ret += id + ", ";
		ret += sequence + ", ";
		ret += "\"" + step + "\"]";

		return ret;
	}

	@Override
	public String getValue() {
		return getStep();
	}

	@Override
	public void setValue(String value, int sequence) {
		setStep(value);
		setSequence(sequence);
	}
}
