package com.zettenet.recipesdb.persistence;

import java.util.List;

import com.zettenet.recipesdb.model.Recipe;
import com.zettenet.recipesdb.model.RecipeCategory;
import com.zettenet.recipesdb.model.RecipeIngredient;
import com.zettenet.recipesdb.model.RecipeIngredientGroup;
import com.zettenet.recipesdb.model.RecipeSource;
import com.zettenet.recipesdb.services.impl.InvalidDataException;

/**
 * A DAO for interacting with objects which are persistent and have a unique
 * "name" field; this includes most of the components of a Recipe.
 * 
 * @author Zetten
 * @param <T>
 *            The underlying class which is handled by the DAO.
 * @see Recipe
 * @see RecipeSource
 * @see RecipeCategory
 * @see RecipeIngredient
 * @see RecipeIngredientGroup
 */
public interface IPersistentUniqueNamedObjectDao<T> {

	/**
	 * Returns all the persisted objects of the given type whose name property
	 * either contains or equals the given string.
	 * 
	 * @param name
	 *            The name of the object to find.
	 * @param inclusive
	 *            If true, returns objects whose name contains the given string.
	 *            If false, returns objects whose name matches the given string.
	 */
	List<T> find(String name, boolean inclusive);

	/**
	 * Returns all the persisted objects of the given type.
	 */
	List<T> findAll();

	/**
	 * Returns the single object persisted with the given identifier.
	 * 
	 * @param id
	 *            The generated ID of the object to be retrieved.
	 */
	T get(Long id);

	/**
	 * Returns a single object with the given unique name. If the object does
	 * not exist in the database, it is saved and then returned. If the object
	 * already exists, it is retrieved and returned.
	 * 
	 * @param name
	 *            The name of the object to be returned.
	 */
	T getOrSave(String name) throws InvalidDataException,
			InstantiationException, IllegalAccessException;

}
