package com.zettenet.recipesdb.persistence;

import java.util.List;
import java.util.Set;

import com.zettenet.recipesdb.model.Recipe;
import com.zettenet.recipesdb.model.RecipeCategory;
import com.zettenet.recipesdb.model.RecipeIngredient;
import com.zettenet.recipesdb.model.RecipeMethod;
import com.zettenet.recipesdb.model.RecipeSource;

/**
 * A DAO for interacting with Recipe objects, providing simple searching and
 * saving capability.
 * 
 * @author Zetten
 * @see Recipe
 * @see IPersistentUniqueNamedObjectDao
 */
public interface IRecipeDao extends IPersistentUniqueNamedObjectDao<Recipe> {

	/**
	 * Returns a list of all the Recipes requiring the specified
	 * RecipeIngredients. Optionally can perform an inclusive or exclusive
	 * search.
	 * 
	 * @param ingredients
	 *            The RecipeIngredients which should be matched.
	 * @param inclusive
	 *            If true, returns Recipes which require one or more of the
	 *            given ingredients. If false, returns only those Recipes which
	 *            require all of the given ingredients.
	 * @return The list of Recipes matching the specified criteria.
	 * @see Recipe
	 * @see RecipeIngredient
	 */
	List<Recipe> findByIngredient(Set<RecipeIngredient> ingredients,
			boolean inclusive);

	/**
	 * Returns a list of all the Recipes for the given source.
	 * 
	 * @param source
	 * @return The list of Recipes matching the specified criteria.
	 * @see Recipe
	 * @see RecipeSource
	 */
	List<Recipe> findBySource(RecipeSource source);

	/**
	 * Returns a list of all the Recipes tagged with the specified
	 * RecipeCategorys. Optionally can perform an inclusive or exclusive search.
	 * 
	 * @param tags
	 * @param inclusive
	 *            If true, returns Recipes which match one or more of the given
	 *            tags. If false, returns only those Recipes which match all of
	 *            the given tags.
	 * @return The list of Recipes matching the specified criteria.
	 * @see Recipe
	 * @see RecipeCategory
	 */
	List<Recipe> findByTag(Set<RecipeCategory> tags, boolean inclusive);

	/**
	 * Saves a new Recipe object to the database, or updates an existing one.
	 * Each component requiring uniqueness is checked and saved independently to
	 * maintain database integrity.
	 * 
	 * @param recipe
	 *            The Recipe to be saved.
	 * @return The database identifier of the saved Recipe.
	 * @see Recipe
	 * @see IPersistentUniqueNamedObjectDao
	 */
	Long saveOrUpdate(Recipe recipe);

	/**
	 * Deletes unneeded records from the database. It will remove:
	 * <ul>
	 *	<li>RecipeCategory objects with no corresponding RecipeCategoryEntry object</li>
	 *	<li>RecipeIngredient objects with no corresponding RecipeIngredientEntry object</li>
	 *	<li>RecipeIngredientGroup objects with no corresponding RecipeIngredientGroup object</li>
	 * </ul>
	 */
	void cleanupDatabase();

	RecipeMethod saveOrUpdate(RecipeMethod method);

}
