package com.zettenet.recipesdb.persistence.hibernate;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.hibernate.Criteria;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.zettenet.recipesdb.model.Recipe;
import com.zettenet.recipesdb.model.RecipeCategory;
import com.zettenet.recipesdb.model.RecipeCategoryEntry;
import com.zettenet.recipesdb.model.RecipeIngredient;
import com.zettenet.recipesdb.model.RecipeIngredientEntry;
import com.zettenet.recipesdb.model.RecipeIngredientGroup;
import com.zettenet.recipesdb.model.RecipeMethod;
import com.zettenet.recipesdb.model.RecipeSource;
import com.zettenet.recipesdb.model.RecipeStep;
import com.zettenet.recipesdb.persistence.IRecipeDao;
import com.zettenet.recipesdb.services.impl.InvalidDataException;

@Repository("recipeDao")
public class RecipeDaoHibernate extends AbstractRecipeComponentDaoHibernate<Recipe> implements IRecipeDao {
	
	public RecipeDaoHibernate() {
		super(Recipe.class);
	}
	
	@Autowired
	public RecipeDaoHibernate(SessionFactory sessionFactory) {
		super(Recipe.class);
		setSessionFactory(sessionFactory);
	}

	@Override
	public List<Recipe> findBySource(RecipeSource source) {
		Criterion crit = Restrictions.eq("source", source);
		return findByCriteria(crit);
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Recipe> findByTag(Set<RecipeCategory> tags, boolean inclusive) {
		Criteria rcpCrit = getSession().createCriteria(Recipe.class);
		Criteria tagCrit = rcpCrit.createCriteria("categories");
		
		if (inclusive) {
			Disjunction disjunction = Restrictions.disjunction();
			
			for (RecipeCategory tag : tags) {
				disjunction.add(Restrictions.eq("category", tag));
			}
			
			tagCrit.add(disjunction);
		} else {
			tagCrit.add(Restrictions.in("category", tags));
		}

		rcpCrit.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
		return rcpCrit.list();
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Recipe> findByIngredient(Set<RecipeIngredient> ingredients,
			boolean inclusive) {
		Criteria rcpCrit = getSession().createCriteria(Recipe.class);
		Criteria ingrCrit = rcpCrit.createCriteria("ingredients");
		
		if (inclusive) {
			Disjunction disjunction = Restrictions.disjunction();
			
			for (RecipeIngredient ingr : ingredients) {
				disjunction.add(Restrictions.eq("ingredient", ingr));
			}
			
			ingrCrit.add(disjunction);
		} else {
			ingrCrit.add(Restrictions.in("ingredient", ingredients));
		}

		rcpCrit.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
		return rcpCrit.list();
	}
	
	@Override
	public Recipe getOrSave(String name) throws InvalidDataException, InstantiationException, IllegalAccessException {
		throw new InvalidDataException("Unable to call getOrSave(String name) for a Recipe object");
	}
	
	@Override
	public Long saveOrUpdate(Recipe recipe) {
		if (recipe.getId() != null && recipe.getId() != 0) {
			for (RecipeStep step : recipe.getMethod().getSteps()) {
				getSession().update(step);
			}
			getSession().update(recipe.getMethod());
			getSession().update(recipe);
		} else {
			getSession().save(recipe);
		}
		return recipe.getId();
	}

	@Override
	public RecipeMethod saveOrUpdate(RecipeMethod method) {
		if (method.getId() != null && method.getId() != 0) {
			List<RecipeStep> tmpSteps = new ArrayList<RecipeStep>();
			
			for (RecipeStep step : method.getSteps()) {
				if (step.getId() != null && step.getId() != 0) {
					RecipeStep tmpStep = (RecipeStep) getSession().get(RecipeStep.class, step.getId());
					tmpStep.setMethod(method);
					tmpStep.setValue(step.getStep(), step.getSequence());
					step = tmpStep;
					getSession().update(step);
				} else {
					getSession().save(step);
				}
				tmpSteps.add(step);
			}
			
			RecipeMethod tmpMethod = (RecipeMethod) getSession().get(RecipeMethod.class, method.getId());
			tmpMethod.setNumberedSteps(method.isNumberedSteps());
			tmpMethod.setPreNote(method.getPreNote());
			tmpMethod.setPostNote(method.getPostNote());
			tmpMethod.setRecipe(method.getRecipe());
			tmpMethod.setSteps(tmpSteps);
			method = tmpMethod;
			getSession().update(method);
		} else {
			getSession().save(method);
		}
		
		return method;
	}

	@Override
	@SuppressWarnings("unchecked")
	public void cleanupDatabase() {
		Criteria allTagEntriesCrit = getSession().createCriteria(RecipeCategoryEntry.class);
		List<RecipeCategoryEntry> usedTagEntries = allTagEntriesCrit.list();
		
		Set<Long> usedTagIds = new HashSet<Long>();
		
		for (RecipeCategoryEntry usedTag : usedTagEntries) {
			usedTagIds.add(usedTag.getCategory().getId());
		}
		
		Criteria unusedTagsCrit = getSession().createCriteria(RecipeCategory.class);
		unusedTagsCrit.add(Restrictions.not(Restrictions.in("id", usedTagIds)));
		unusedTagsCrit.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
		
		Criteria allIngrEntriesCrit = getSession().createCriteria(RecipeIngredientEntry.class);
		List<RecipeIngredientEntry> usedIngrEntries = allIngrEntriesCrit.list();
		
		Set<Long> usedIngrIds = new HashSet<Long>();
		Set<Long> usedIngrGrpIds = new HashSet<Long>();
		
		for (RecipeIngredientEntry usedIngr : usedIngrEntries) {
			usedIngrIds.add(usedIngr.getIngredient().getId());
			usedIngrGrpIds.add(usedIngr.getGroup().getId());
		}
		
		Criteria unusedIngrsCrit = getSession().createCriteria(RecipeIngredient.class);
		unusedIngrsCrit.add(Restrictions.not(Restrictions.in("id", usedIngrIds)));
		unusedIngrsCrit.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);

		Criteria unusedIngrGrpsCrit = getSession().createCriteria(RecipeIngredientGroup.class);
		unusedIngrGrpsCrit.add(Restrictions.not(Restrictions.in("id", usedIngrGrpIds)));
		unusedIngrGrpsCrit.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
		
		/****************/
		
		for (Object objToDel : unusedTagsCrit.list()) {
			getSession().delete(objToDel);
		}
		
		for (Object objToDel : unusedIngrsCrit.list()) {
			getSession().delete(objToDel);
		}
		
		for (Object objToDel : unusedIngrGrpsCrit.list()) {
			getSession().delete(objToDel);
		}
	}

}
