package com.zettenet.recipesdb.services.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.zettenet.recipesdb.model.Recipe;
import com.zettenet.recipesdb.model.RecipeCategory;
import com.zettenet.recipesdb.model.RecipeCategoryEntry;
import com.zettenet.recipesdb.model.RecipeIngredient;
import com.zettenet.recipesdb.model.RecipeIngredientEntry;
import com.zettenet.recipesdb.model.RecipeIngredientGroup;
import com.zettenet.recipesdb.model.RecipeSource;
import com.zettenet.recipesdb.persistence.IPersistentUniqueNamedObjectDao;
import com.zettenet.recipesdb.persistence.IRecipeDao;
import com.zettenet.recipesdb.services.IRecipesDataService;

@Service("recipesDataService")
public class RecipesDataServiceImpl implements IRecipesDataService {

	private final Logger LOGGER = LoggerFactory.getLogger(getClass());
	public static final int DEFAULT_TOKEN_LENGTH = 10;

	@Resource
	private IRecipeDao recipeDao;
	@Resource
	private IPersistentUniqueNamedObjectDao<RecipeSource> recipeSourceDao;
	@Resource
	private IPersistentUniqueNamedObjectDao<RecipeIngredient> recipeIngredientDao;
	@Resource
	private IPersistentUniqueNamedObjectDao<RecipeIngredientGroup> recipeIngredientGroupDao;
	@Resource
	private IPersistentUniqueNamedObjectDao<RecipeCategory> recipeCategoryDao;
	private int tokenLength;

	public RecipesDataServiceImpl() {
		this.tokenLength = DEFAULT_TOKEN_LENGTH;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void cleanupDatabase(String clientToken) throws InvalidClientException {
		this.LOGGER.debug("cleanupDatabase: clientToken = {}", clientToken);
		validateClient(clientToken);
		recipeDao.cleanupDatabase();
	}

	@Override
	@Transactional(readOnly = true)
	public List<RecipeIngredient> getAllIngredients(String clientToken)
			throws InvalidClientException {
		this.LOGGER.debug("getAllIngredients: clientToken = {}", clientToken);
		validateClient(clientToken);

		List<RecipeIngredient> results = this.recipeIngredientDao.findAll();

		if ((results == null) || results.isEmpty()) {
			this.LOGGER.warn("No Recipe Ingredients found in database");
			results = new ArrayList<RecipeIngredient>();
		}

		this.LOGGER.debug("getAllIngredients: found {} ingredient{}",
				results.size(), results.size() == 1 ? "" : "s");

		return results;
	}

	@Override
	@Transactional(readOnly = true)
	public List<Recipe> getAllRecipes(String clientToken)
			throws InvalidClientException {
		this.LOGGER.debug("getAllRecipes: clientToken = {}", clientToken);
		validateClient(clientToken);

		List<Recipe> results = this.recipeDao.findAll();

		if ((results == null) || results.isEmpty()) {
			this.LOGGER.warn("No Recipes found in database");
			results = new ArrayList<Recipe>();
		}

		this.LOGGER.debug("getAllRecipes: found {} recipe{}", results.size(),
				results.size() == 1 ? "" : "s");

		return results;
	}

	@Override
	@Transactional(readOnly = true)
	public List<RecipeSource> getAllSources(String clientToken)
			throws InvalidClientException {
		this.LOGGER.debug("getAllSources: clientToken = {}", clientToken);
		validateClient(clientToken);

		List<RecipeSource> results = this.recipeSourceDao.findAll();

		if ((results == null) || results.isEmpty()) {
			this.LOGGER.warn("No Recipe Sources found in database");
			results = new ArrayList<RecipeSource>();
		}

		this.LOGGER.debug("getAllSources: found {} source{}", results.size(),
				results.size() == 1 ? "" : "s");

		return results;
	}

	@Override
	@Transactional(readOnly = true)
	public List<RecipeCategory> getAllTags(String clientToken)
			throws InvalidClientException {
		this.LOGGER.debug("getAllTags: clientToken = {}", clientToken);
		validateClient(clientToken);

		List<RecipeCategory> results = this.recipeCategoryDao.findAll();

		if ((results == null) || results.isEmpty()) {
			this.LOGGER.warn("No Recipe Categories found in database");
			results = new ArrayList<RecipeCategory>();
		}

		this.LOGGER.debug("getAllTags: found {} tag{}", results.size(),
				results.size() == 1 ? "" : "s");

		return results;
	}

	@Override
	@Transactional(readOnly = true)
	public RecipeIngredient getIngredient(String clientToken, Long id)
			throws InvalidClientException, InvalidDataException {
		this.LOGGER.debug("getIngredient: clientToken = {}, id = {}",
				clientToken, id);
		validateClient(clientToken);

		RecipeIngredient result = this.recipeIngredientDao.get(id);

		if (result == null) {
			this.LOGGER.error("No Ingredient exists for the requested ID ({})",
					id);
			throw new InvalidDataException(
					"No Ingredient exists for the requested ID (" + id + ")");
		}

		this.LOGGER.debug("getIngredient: returning \"{}\"", result.getName());

		return result;
	}

	@Override
	@Transactional(readOnly = true)
	public RecipeIngredientGroup getIngredientGroup(String clientToken, Long id)
			throws InvalidClientException, InvalidDataException {
		this.LOGGER.debug("getIngredientGroup: clientToken = {}, id = {}",
				clientToken, id);
		validateClient(clientToken);

		RecipeIngredientGroup result = this.recipeIngredientGroupDao.get(id);

		if (result == null) {
			this.LOGGER.error(
					"No Ingredient Group exists for the requested ID ({})", id);
			throw new InvalidDataException(
					"No Ingredient Group exists for the requested ID (" + id
							+ ")");
		}

		this.LOGGER.debug("getIngredientGroup: returning \"{}\"",
				result.getName());

		return result;
	}

	@Override
	@Transactional(readOnly = true)
	public List<RecipeIngredient> getIngredientsByName(String clientToken,
			String name) throws InvalidClientException {
		return getIngredientsByName(clientToken, name, true);
	}

	@Override
	@Transactional(readOnly = true)
	public List<RecipeIngredient> getIngredientsByName(String clientToken,
			String name, boolean inclusive) throws InvalidClientException {
		this.LOGGER
				.debug("getIngredientsByName: clientToken = {}, name = {}, inclusive = {}",
						new Object[] { clientToken, name, inclusive });
		validateClient(clientToken);

		List<RecipeIngredient> results = this.recipeIngredientDao.find(name,
				inclusive);

		if (results == null) {
			results = new ArrayList<RecipeIngredient>();
		}

		this.LOGGER.debug("getIngredientsByName: found {} ingredient{}",
				results.size(), results.size() == 1 ? "" : "s");

		return results;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public RecipeIngredient getOrSaveIngredient(String clientToken, String name)
			throws InvalidClientException, InvalidDataException,
			InstantiationException, IllegalAccessException {
		this.LOGGER.debug("getOrSaveIngredient: clientToken = {}, name = {}",
				clientToken, name);
		validateClient(clientToken);

		RecipeIngredient result = this.recipeIngredientDao.getOrSave(name);

		this.LOGGER.debug("getOrSaveIngredient: returning \"{}\"",
				result.getName());

		return result;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public RecipeIngredientGroup getOrSaveIngredientGroup(String clientToken,
			String name) throws InvalidClientException, InvalidDataException,
			InstantiationException, IllegalAccessException {
		this.LOGGER.debug(
				"getOrSaveIngredientGroup: clientToken = {}, name = {}",
				clientToken, name);
		validateClient(clientToken);

		RecipeIngredientGroup result = this.recipeIngredientGroupDao
				.getOrSave(name);

		this.LOGGER.debug("getOrSaveIngredientGroup: returning \"{}\"",
				result.getName());

		return result;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public RecipeSource getOrSaveSource(String clientToken, String name)
			throws InvalidClientException, InvalidDataException,
			InstantiationException, IllegalAccessException {
		this.LOGGER.debug("getOrSaveSource: clientToken = {}, name = {}",
				clientToken, name);
		validateClient(clientToken);

		RecipeSource result = this.recipeSourceDao.getOrSave(name);

		this.LOGGER
				.debug("getOrSaveSource: returning \"{}\"", result.getName());

		return result;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public RecipeCategory getOrSaveTag(String clientToken, String name)
			throws InvalidClientException, InvalidDataException,
			InstantiationException, IllegalAccessException {
		this.LOGGER.debug("getOrSaveTag: clientToken = {}, name = {}",
				clientToken, name);
		validateClient(clientToken);

		RecipeCategory result = this.recipeCategoryDao.getOrSave(name);

		this.LOGGER.debug("getOrSaveTag: returning \"{}\"", result.getName());

		return result;
	}

	@Override
	@Transactional(readOnly = true)
	public Recipe getRecipe(String clientToken, Long id)
			throws InvalidClientException, InvalidDataException {
		this.LOGGER.debug("getRecipe: clientToken = {}, id = {}", clientToken,
				id);
		validateClient(clientToken);

		Recipe result = this.recipeDao.get(id);

		if (result == null) {
			this.LOGGER.error("No Recipe exists for the requested ID ({})", id);
			throw new InvalidDataException(
					"No Recipe exists for the requested ID (" + id + ")");
		}

		this.LOGGER.debug("getRecipe: returning \"{}\"", result.getName());

		return result;
	}

	@Override
	public List<Recipe> getRecipesByIngredient(String clientToken,
			Set<RecipeIngredient> ingredients) throws InvalidClientException {
		return getRecipesByIngredient(clientToken, ingredients, false);
	}

	@Override
	@Transactional(readOnly = true)
	public List<Recipe> getRecipesByIngredient(String clientToken,
			Set<RecipeIngredient> ingredients, boolean inclusive)
			throws InvalidClientException {
		this.LOGGER
				.debug("getRecipesByIngredient: clientToken = {}, ingredients = {}, inclusive = {}",
						new Object[] { clientToken, ingredients, inclusive });
		validateClient(clientToken);
		
		Set<RecipeIngredient> persistentIngrs = new HashSet<RecipeIngredient>();
		
		for (RecipeIngredient ingr : ingredients) {
			if (ingr.getId() == null) {
				List<RecipeIngredient> tmpIngrs = getIngredientsByName(clientToken, ingr.getName());
				if (tmpIngrs != null && !tmpIngrs.isEmpty()) {
					persistentIngrs.add(tmpIngrs.get(0));
				}
			} else {
				persistentIngrs.add(ingr);
			}
		}

		if (!persistentIngrs.isEmpty()) {
			List<Recipe> results = this.recipeDao.findByIngredient(persistentIngrs,
					inclusive);

			this.LOGGER.debug("getRecipesByIngredient: found {} recipe{}",
					results.size(), results.size() == 1 ? "" : "s");

			return results;
		}

		this.LOGGER.warn("getRecipesByIngredient: no persistent ingredients supplied");
		
		return new ArrayList<Recipe>();
	}

	@Override
	@Transactional(readOnly = true)
	public List<Recipe> getRecipesByName(String clientToken, String name)
			throws InvalidClientException {
		return getRecipesByName(clientToken, name, true);
	}

	@Override
	@Transactional(readOnly = true)
	public List<Recipe> getRecipesByName(String clientToken, String name,
			boolean inclusive) throws InvalidClientException {
		this.LOGGER
				.debug("getRecipesByName: clientToken = {}, name = {}, inclusive = {}",
						new Object[] { clientToken, name, inclusive });
		validateClient(clientToken);

		List<Recipe> results = this.recipeDao.find(name, inclusive);

		if (results == null) {
			results = new ArrayList<Recipe>();
		}

		this.LOGGER.debug("getRecipesByName: found {} recipe{}",
				results.size(), results.size() == 1 ? "" : "s");

		return results;
	}

	@Override
	@Transactional(readOnly = true)
	public List<Recipe> getRecipesBySource(String clientToken,
			RecipeSource source) throws InvalidClientException {
		this.LOGGER.debug("getRecipesBySource: clientToken = {}, source = {}",
				clientToken, source);
		validateClient(clientToken);
		
		RecipeSource persistentSource = null;
		
		if (source.getId() == null) {
			List<RecipeSource> tmpSrcs = getSourcesByName(clientToken, source.getName());
			if (tmpSrcs != null && !tmpSrcs.isEmpty()) {
				persistentSource = tmpSrcs.get(0);
			}
		}

		if (persistentSource != null) {
			List<Recipe> results = this.recipeDao.findBySource(persistentSource);
	
			this.LOGGER.debug("getRecipesBySource: found {} recipe{}",
					results.size(), results.size() == 1 ? "" : "s");
	
			return results;
		}

		this.LOGGER.warn("getRecipesBySource: no persistent source supplied");
		
		return new ArrayList<Recipe>();
	}

	@Override
	@Transactional(readOnly = true)
	public List<Recipe> getRecipesByTag(String clientToken,
			Set<RecipeCategory> tags) throws InvalidClientException {
		return getRecipesByTag(clientToken, tags, false);
	}

	@Override
	@Transactional(readOnly = true)
	public List<Recipe> getRecipesByTag(String clientToken,
			Set<RecipeCategory> tags, boolean inclusive)
			throws InvalidClientException {
		this.LOGGER.debug(
				"getRecipesByTag: clientToken = {}, tags = {}, inclusive = {}",
				new Object[] { clientToken, tags, inclusive });
		validateClient(clientToken);
		
		Set<RecipeCategory> persistentTags = new HashSet<RecipeCategory>();
		
		for (RecipeCategory tag : tags) {
			if (tag.getId() == null) {
				List<RecipeCategory> tmpTags = getTagsByName(clientToken, tag.getName());
				if (tmpTags != null && !tmpTags.isEmpty()) {
					persistentTags.add(tmpTags.get(0));
				}
			} else {
				persistentTags.add(tag);
			}
		}

		if (!persistentTags.isEmpty()) {
			List<Recipe> results = this.recipeDao.findByTag(persistentTags, inclusive);
	
			this.LOGGER.debug("getRecipesByTag: found {} recipe{}", results.size(),
					results.size() == 1 ? "" : "s");
	
			return results;
		}

		this.LOGGER.warn("getRecipesByTag: no persistent tags supplied");
		
		return new ArrayList<Recipe>();
	}

	@Override
	@Transactional(readOnly = true)
	public RecipeSource getSource(String clientToken, Long id)
			throws InvalidClientException, InvalidDataException {
		this.LOGGER.debug("getSource: clientToken = {}, id = {}", clientToken,
				id);
		validateClient(clientToken);

		RecipeSource result = this.recipeSourceDao.get(id);

		if (result == null) {
			this.LOGGER.error("No Source exists for the requested ID ({})", id);
			throw new InvalidDataException(
					"No Source exists for the requested ID (" + id + ")");
		}

		this.LOGGER.debug("getSource: returning \"{}\"", result.getName());

		return result;
	}

	@Override
	@Transactional(readOnly = true)
	public List<RecipeSource> getSourcesByName(String clientToken, String name)
			throws InvalidClientException {
		return getSourcesByName(clientToken, name, true);
	}

	@Override
	@Transactional(readOnly = true)
	public List<RecipeSource> getSourcesByName(String clientToken, String name,
			boolean inclusive) throws InvalidClientException {
		this.LOGGER
				.debug("getSourcesByName: clientToken = {}, name = {}, inclusive = {}",
						new Object[] { clientToken, name, inclusive });
		validateClient(clientToken);

		List<RecipeSource> results = this.recipeSourceDao.find(name, inclusive);

		if (results == null) {
			results = new ArrayList<RecipeSource>();
		}

		this.LOGGER.debug("getSourcesByName: found {} source{}",
				results.size(), results.size() == 1 ? "" : "s");

		return results;
	}

	@Override
	@Transactional(readOnly = true)
	public RecipeCategory getTag(String clientToken, Long id)
			throws InvalidClientException, InvalidDataException {
		this.LOGGER.debug("getTag: clientToken = {}, id = {}", clientToken, id);
		validateClient(clientToken);

		RecipeCategory result = this.recipeCategoryDao.get(id);

		if (result == null) {
			this.LOGGER.error("No Tag exists for the requested ID ({})", id);
			throw new InvalidDataException(
					"No Tag exists for the requested ID (" + id + ")");
		}

		this.LOGGER.debug("getTag: returning \"{}\"", result.getName());

		return result;
	}

	@Override
	@Transactional(readOnly = true)
	public List<RecipeCategory> getTagsByName(String clientToken, String name)
			throws InvalidClientException {
		return getTagsByName(clientToken, name, true);
	}

	@Override
	@Transactional(readOnly = true)
	public List<RecipeCategory> getTagsByName(String clientToken, String name,
			boolean inclusive) throws InvalidClientException {
		this.LOGGER.debug(
				"getTagsByName: clientToken = {}, name = {}, inclusive = {}",
				new Object[] { clientToken, name, inclusive });
		validateClient(clientToken);

		List<RecipeCategory> results = this.recipeCategoryDao.find(name,
				inclusive);

		if (results == null) {
			results = new ArrayList<RecipeCategory>();
		}

		this.LOGGER.debug("getTagsByName: found {} tag{}", results.size(),
				results.size() == 1 ? "" : "s");

		return results;
	}

	/**
	 * @return the tokenLength
	 */
	public int getTokenLength() {
		return this.tokenLength;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public Long saveOrUpdateRecipe(String clientToken, Recipe recipe)
			throws InvalidClientException, InvalidDataException,
			InstantiationException, IllegalAccessException {
		this.LOGGER.debug("saveOrUpdateRecipe: clientToken = {}, name = {}",
				clientToken, recipe.getName());
		validateClient(clientToken);

		recipe.setSource(getOrSaveSource(clientToken, recipe.getSource()
				.getName()));

		for (RecipeIngredientEntry ingrEntry : recipe.getIngredients()) {
			ingrEntry.setIngredient(getOrSaveIngredient(clientToken, ingrEntry
					.getIngredient().getName()));
			ingrEntry.setGroup(getOrSaveIngredientGroup(clientToken, ingrEntry
					.getGroup().getName()));
		}

		for (RecipeCategoryEntry catEntry : recipe.getCategories()) {
			catEntry.setCategory(getOrSaveTag(clientToken, catEntry
					.getCategory().getName()));
		}

		Long result = this.recipeDao.saveOrUpdate(recipe);

		this.LOGGER.debug("saveOrUpdateRecipe: returning id {}", result);

		return result;
	}

	/**
	 * @param tokenLength
	 *            the tokenLength to set
	 */
	public void setTokenLength(int tokenLength) {
		this.tokenLength = tokenLength;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void tagRecipes(String clientToken, RecipeCategory tag,
			Set<Recipe> recipes) throws InvalidClientException,
			InvalidDataException, InstantiationException,
			IllegalAccessException {
		this.LOGGER.debug("tagRecipes: clientToken = {}, tag = {}",
				clientToken, tag.getName());
		validateClient(clientToken);

		tag = getOrSaveTag(clientToken, tag.getName());

		for (Recipe recipe : recipes) {
			if (recipe.getId() != null) {
				this.LOGGER.debug("tagRecipes: tagging recipe {}",
						recipe.getId());
				recipe = getRecipe(clientToken, recipe.getId());
				recipe.addCategory(tag);
				saveOrUpdateRecipe(clientToken, recipe);
			}
		}

		this.LOGGER
				.debug("tagRecipes: done; tagged {} recipes", recipes.size());
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void untagRecipes(String clientToken, RecipeCategory tag,
			Set<Recipe> recipes) throws InvalidClientException,
			InvalidDataException, InstantiationException,
			IllegalAccessException {
		this.LOGGER.debug("untagRecipes: clientToken = {}, tag = {}",
				clientToken, tag.getName());
		validateClient(clientToken);

		tag = getOrSaveTag(clientToken, tag.getName());

		for (Recipe recipe : recipes) {
			if (recipe.getId() != null) {
				this.LOGGER.debug("untagRecipes: untagging recipe {}",
						recipe.getId());
				recipe = getRecipe(clientToken, recipe.getId());
				recipe.removeCategory(tag);
				saveOrUpdateRecipe(clientToken, recipe);
			}
		}

		this.LOGGER.debug("tagRecipes: done; untagged {} recipes",
				recipes.size());
	}

	private void validateClient(String clientToken)
			throws InvalidClientException {
		// TODO Auto-generated method stub
	}

}
