package com.zettenet.recipesdb.test;

import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.zettenet.recipesdb.calculator.Calculator;
import com.zettenet.recipesdb.calculator.Constants.*;

public class TestCalculator {
	private static final MathContext mc = new MathContext(6, RoundingMode.HALF_UP);
	
	private static final BigDecimal expectedTbspToMl = new BigDecimal("29.5735296").round(mc).stripTrailingZeros();
	private static final BigDecimal expectedFlozToMl = new BigDecimal("118.294118").round(mc).stripTrailingZeros();
	private static final BigDecimal expectedCupsToPints = new BigDecimal("1").round(mc).stripTrailingZeros();
	
	private static final BigDecimal expectedOzToGrams = new BigDecimal("113.398093").round(mc).stripTrailingZeros();
	private static final BigDecimal expectedLbToOz = new BigDecimal("32").round(mc).stripTrailingZeros();
	
	private static final BigDecimal expectedFlozWaterToGrams = new BigDecimal("147.867648").round(mc).stripTrailingZeros();
	private static final BigDecimal expectedCupsFlourToGrams = new BigDecimal("249.837178").round(mc).stripTrailingZeros();
	private static final BigDecimal expectedTbspOilToGrams = new BigDecimal("54.2970003").round(mc).stripTrailingZeros();
	
	private static final BigDecimal expectedGramsWaterToFloz = new BigDecimal("16.9070114").round(mc).stripTrailingZeros();
	private static final BigDecimal expectedGramsFlourToCups = new BigDecimal("2").round(mc).stripTrailingZeros();
	private static final BigDecimal expectedOzSaltToTsp = new BigDecimal("9.9682296").round(mc).stripTrailingZeros();
	
	private BigDecimal amt = null;
	private MeasureVolume srcVol = null;
	private MeasureWeight srcWgt = null;
	private Ingredient ing = null;
	private MeasureVolume tarVol = null;
	private MeasureWeight tarWgt = null;
	
	BigDecimal result = null;
	BigDecimal expResult = null;

	@Test (description = "Volume to volume conversion is correct.")
	public void testCalculateVolumeToVolume() {
		// Calculate 2 tablespoons in ml
		amt = new BigDecimal("2");
		srcVol = MeasureVolume.TABLESPOON;
		tarVol = MeasureVolume.MILLILITRE;
		expResult = expectedTbspToMl;
		
		result = Calculator.calculate(amt, srcVol, tarVol).round(mc).stripTrailingZeros();
		Assert.assertEquals(expResult,
				result,
				"Expected tbsp to ml in a ratio of 14.787:");

		// Calculate 4 fluid ounces in ml
		amt = new BigDecimal("4");
		srcVol = MeasureVolume.FLUID_OUNCE;
		tarVol = MeasureVolume.MILLILITRE;
		expResult = expectedFlozToMl;
		
		result = Calculator.calculate(amt, srcVol, tarVol).round(mc).stripTrailingZeros();
		Assert.assertEquals(expResult,
				result,
				"Expected fl.oz. to ml in a ratio of 29.574:");

		// Calculate 2 US cups in US liquid pints
		amt = new BigDecimal("2");
		srcVol = MeasureVolume.CUP_US;
		tarVol = MeasureVolume.PINT_US_LIQUID;
		expResult = expectedCupsToPints;
		
		result = Calculator.calculate(amt, srcVol, tarVol).round(mc).stripTrailingZeros();
		Assert.assertEquals(expResult,
				result,
				"Expected cups to pints in a ratio of 2:");
	}

	@Test (description = "Weight to weight conversion is correct.")
	public void testCalculateWeightToWeight() {
		// Calculate 4 ounces in grams
		amt = new BigDecimal("4");
		srcWgt = MeasureWeight.OUNCE;
		tarWgt = MeasureWeight.GRAM;
		expResult = expectedOzToGrams;
		
		result = Calculator.calculate(amt, srcWgt, tarWgt).round(mc).stripTrailingZeros();
		Assert.assertEquals(expResult,
				result,
				"Expected oz. to grams in a ratio of 28.35:");

		// Calculate 2 pounds in ounces
		amt = new BigDecimal("2");
		srcWgt = MeasureWeight.POUND;
		tarWgt = MeasureWeight.OUNCE;
		expResult = expectedLbToOz;
		
		result = Calculator.calculate(amt, srcWgt, tarWgt).round(mc).stripTrailingZeros();
		Assert.assertEquals(expResult,
				result,
				"Expected lb. to oz. in a ratio of 16:");
	}

	@Test (description = "Volume to weight conversion with ingredient density is correct.")
	public void testCalculateVolumeToWeight() {
		// Calculate 5 fluid ounces of water in grams
		amt = new BigDecimal("5");
		srcVol = MeasureVolume.FLUID_OUNCE;
		ing = Ingredient.WATER;
		tarWgt = MeasureWeight.GRAM;
		expResult = expectedFlozWaterToGrams;

		result = Calculator.calculate(amt, srcVol, ing, tarWgt).round(mc).stripTrailingZeros();
		Assert.assertEquals(expResult,
				result,
				"Expected fl. oz. of water to grams in a ratio of 29.57:");
		
		// Calculate 2 cups of flour in grams
		amt = new BigDecimal("2");
		srcVol = MeasureVolume.CUP_US;
		ing = Ingredient.FLOUR_WHITE;
		tarWgt = MeasureWeight.GRAM;
		expResult = expectedCupsFlourToGrams;
		
		result = Calculator.calculate(amt, srcVol, ing, tarWgt).round(mc).stripTrailingZeros();
		Assert.assertEquals(expResult,
				result,
				"Expected cups of flour to grams in a ratio of 249.84:");
		
		// Calculate 4 tablespoons of olive oil in grams
		amt = new BigDecimal("4");
		srcVol = MeasureVolume.TABLESPOON;
		ing = Ingredient.OIL_OLIVE;
		tarWgt = MeasureWeight.GRAM;
		expResult = expectedTbspOilToGrams;
		
		result = Calculator.calculate(amt, srcVol, ing, tarWgt).round(mc).stripTrailingZeros();
		Assert.assertEquals(expResult,
				result,
				"Expected tbsp of oil to grams in a ratio of 13.57:");
	}

	@Test (description = "Weight to volume conversion with ingredient density is correct.")
	public void testCalculateWeightToVolume() {
		// Calculate 500 grams of water in fluid ounces
		amt = new BigDecimal("500");
		srcWgt = MeasureWeight.GRAM;
		ing = Ingredient.WATER;
		tarVol = MeasureVolume.FLUID_OUNCE;
		expResult = expectedGramsWaterToFloz;

		result = Calculator.calculate(amt, srcWgt, ing, tarVol).round(mc).stripTrailingZeros();
		Assert.assertEquals(expResult,
				result,
				"Expected grams of water to fl. oz. in a ratio of 0.0338:");
		
		// Calculate 250 grams of flour in US cups
		amt = new BigDecimal("249.837178").round(mc).stripTrailingZeros();
		srcWgt = MeasureWeight.GRAM;
		ing = Ingredient.FLOUR_WHITE;
		tarVol = MeasureVolume.CUP_US;
		expResult = expectedGramsFlourToCups;

		result = Calculator.calculate(amt, srcWgt, ing, tarVol).round(mc).stripTrailingZeros();
		Assert.assertEquals(expResult,
				result,
				"Expected grams of flour to US cups in a ratio of 125:");
		
		// Calculate 2 ounces of salt in teaspoons
		amt = new BigDecimal("2");
		srcWgt = MeasureWeight.OUNCE;
		ing = Ingredient.SALT_TABLE;
		tarVol = MeasureVolume.TEASPOON;
		expResult = expectedOzSaltToTsp;

		result = Calculator.calculate(amt, srcWgt, ing, tarVol).round(mc).stripTrailingZeros();
		Assert.assertEquals(expResult,
				result,
				"Expected ounces of salt to a teaspoon in a ratio of 4.984:");
	}

}
