package com.zettenet.recipesdb.test;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.zettenet.recipesdb.model.Recipe;
import com.zettenet.recipesdb.model.RecipeCategory;
import com.zettenet.recipesdb.model.RecipeCategoryEntry;
import com.zettenet.recipesdb.model.RecipeIngredient;
import com.zettenet.recipesdb.model.RecipeIngredientGroup;
import com.zettenet.recipesdb.model.RecipeSource;
import com.zettenet.recipesdb.services.IRecipesDataService;
import com.zettenet.recipesdb.test.harness.RecipeTestData;


public class TestRecipesDataService extends TestRecipesApplication {
	
	private IRecipesDataService svc;
	private final String clientToken = "test";
	
	RecipeCategory tagTest = new RecipeCategory();
	RecipeCategory tagWrongTest;
	RecipeCategory tagRightTest;
	RecipeSource srcTest = new RecipeSource();
	RecipeIngredient ingrTest = new RecipeIngredient();
	RecipeIngredientGroup grpTest = new RecipeIngredientGroup();
	RecipeCategory tagNew = new RecipeCategory();
	RecipeSource srcNew = new RecipeSource();
	RecipeIngredient ingrNew = new RecipeIngredient();
	Recipe rcpTest = RecipeTestData.getFullRecipe();

	@BeforeClass
	public void setUpService() {
		svc = (IRecipesDataService) ctx.getBean("recipesDataService");
	}
	
	@Test(description = "Verifies that no methods fail completely when data does not exist")
	public void testEmptyDatabase() {
		List<RecipeCategory> noTags = Collections.emptyList();
		List<RecipeSource> noSrcs = Collections.emptyList();
		List<RecipeIngredient> noIngrs = Collections.emptyList();
		List<Recipe> noRcps = Collections.emptyList();
		
		try {
			noTags = svc.getAllTags(clientToken);
			noSrcs = svc.getAllSources(clientToken);
			noIngrs = svc.getAllIngredients(clientToken);
			noRcps = svc.getAllRecipes(clientToken);
		} catch (Exception ex) {
			Assert.fail("Failed to 'retrieve all' when no data is stored", ex);
		}
		
		Assert.assertTrue(noTags.isEmpty());
		Assert.assertTrue(noSrcs.isEmpty());
		Assert.assertTrue(noIngrs.isEmpty());
		Assert.assertTrue(noRcps.isEmpty());
	}
	
	@Test(description = "Verifies that unique-named objects can be saved and retrieved correctly",
			dependsOnMethods = "testEmptyDatabase")
	public void testGetOrSave() {
		// Save new components
		try {
			tagTest = svc.getOrSaveTag(clientToken, RecipeTestData.catMain.getName());
			srcTest = svc.getOrSaveSource(clientToken, RecipeTestData.srcNewspaper.getName());
			ingrTest = svc.getOrSaveIngredient(clientToken, RecipeTestData.ingrFlour.getName());
			grpTest = svc.getOrSaveIngredientGroup(clientToken, RecipeTestData.ingrGroupMain.getName());
		} catch (Exception ex) {
			Assert.fail("Failed to save a new unique-named component", ex);
		}
		
		Assert.assertNotNull(tagTest.getId());
		Assert.assertNotNull(srcTest.getId());
		Assert.assertNotNull(ingrTest.getId());
		Assert.assertNotNull(grpTest.getId());
		
		RecipeCategory tagDupe = new RecipeCategory();
		RecipeSource srcDupe = new RecipeSource();
		RecipeIngredient ingrDupe = new RecipeIngredient();
		RecipeIngredientGroup grpDupe = new RecipeIngredientGroup();
		
		try {
			tagDupe = svc.getOrSaveTag(clientToken, RecipeTestData.catMain.getName());
			srcDupe = svc.getOrSaveSource(clientToken, RecipeTestData.srcNewspaper.getName());
			ingrDupe = svc.getOrSaveIngredient(clientToken, RecipeTestData.ingrFlour.getName());
			grpDupe = svc.getOrSaveIngredientGroup(clientToken, RecipeTestData.ingrGroupMain.getName());
		} catch (Exception ex) {
			Assert.fail("Failed to retrieve a saved unique-named component", ex);
		}
		
		Assert.assertEquals(tagDupe.getId(), tagTest.getId());
		Assert.assertEquals(srcDupe.getId(), srcTest.getId());
		Assert.assertEquals(ingrDupe.getId(), ingrTest.getId());
		Assert.assertEquals(grpDupe.getId(), grpTest.getId());
	}
	
	@Test(description = "Verifies that we can retrieve persistent objects by ID",
			dependsOnMethods = "testGetOrSave")
	public void testGetById() {
		RecipeCategory tagIdTest = new RecipeCategory();
		RecipeSource srcIdTest = new RecipeSource();
		RecipeIngredient ingrIdTest = new RecipeIngredient();
		RecipeIngredientGroup grpIdTest = new RecipeIngredientGroup();
		
		try {
			tagIdTest = svc.getTag(clientToken, tagTest.getId());
			srcIdTest = svc.getSource(clientToken, srcTest.getId());
			ingrIdTest = svc.getIngredient(clientToken, ingrTest.getId());
			grpIdTest = svc.getIngredientGroup(clientToken, grpTest.getId());
		} catch (Exception ex) {
			Assert.fail("Failed to retrieve a component by ID", ex);
		}
		
		Assert.assertEquals(tagIdTest, tagTest);
		Assert.assertEquals(srcIdTest, srcTest);
		Assert.assertEquals(ingrIdTest, ingrTest);
		Assert.assertEquals(grpIdTest, grpTest);
	}
	
	@Test(description = "Verifies that retrieving all instances of a component occurs correctly",
			dependsOnMethods = "testGetOrSave")
	public void testGetAll() {
		List<RecipeCategory> tagAll = Collections.emptyList();
		List<RecipeSource> srcAll = Collections.emptyList();
		List<RecipeIngredient> ingrAll = Collections.emptyList();
		
		try {
			tagAll = svc.getAllTags(clientToken);
			srcAll = svc.getAllSources(clientToken);
			ingrAll = svc.getAllIngredients(clientToken);
		} catch (Exception ex) {
			Assert.fail("Failed to retrieve all components", ex);
		}
		
		Assert.assertTrue(tagAll.size() == 1);
		Assert.assertTrue(srcAll.size() == 1);
		Assert.assertTrue(ingrAll.size() == 1);
		
		try {
			tagNew = svc.getOrSaveTag(clientToken, RecipeTestData.catDessert.getName());
			srcNew = svc.getOrSaveSource(clientToken, RecipeTestData.srcMagazine.getName());
			ingrNew = svc.getOrSaveIngredient(clientToken, RecipeTestData.ingrEggs.getName());
			
			tagAll = svc.getAllTags(clientToken);
			srcAll = svc.getAllSources(clientToken);
			ingrAll = svc.getAllIngredients(clientToken);
		} catch (Exception ex) {
			Assert.fail("Failed to save a new unique-named component", ex);
		}
		
		Assert.assertTrue(tagAll.size() == 2);
		Assert.assertTrue(tagAll.contains(tagNew));
		Assert.assertTrue(srcAll.size() == 2);
		Assert.assertTrue(srcAll.contains(srcNew));
		Assert.assertTrue(ingrAll.size() == 2);
		Assert.assertTrue(ingrAll.contains(ingrNew));
	}
	
	@Test(description = "Verifies that the inclusive/exclusive search by name works correctly",
			dependsOnMethods = "testGetAll")
	public void testGetByName() {
		RecipeCategory tagNew = new RecipeCategory();
		
		try {
			tagNew = svc.getOrSaveTag(clientToken, RecipeTestData.catMainCat.getName());
		} catch (Exception ex) {
			Assert.fail("Failed to save a new unique-named Tag", ex);
		}
		
		Assert.assertFalse(tagNew.getId() == null);
		
		List<RecipeCategory> tagsIncl = Collections.emptyList();
		List<RecipeCategory> tagsExcl = Collections.emptyList();
		List<RecipeCategory> tagsNone = Collections.emptyList();
		
		try {
			tagsIncl = svc.getTagsByName(clientToken, "Main course", true);
			tagsExcl = svc.getTagsByName(clientToken, "Main course", false);
			tagsNone = svc.getTagsByName(clientToken, "This tag is obviously fake");
		} catch (Exception ex) {
			Assert.fail("Failed to get Tags by name", ex);
		}

		Assert.assertTrue(tagsExcl.size() == 1);
		Assert.assertTrue(tagsExcl.contains(tagTest));
		
		Assert.assertTrue(tagsIncl.size() == 2);
		Assert.assertTrue(tagsIncl.contains(tagTest));
		Assert.assertTrue(tagsIncl.contains(tagNew));
		
		Assert.assertTrue(tagsNone.isEmpty());
	}
	
	@Test(description = "Verifies that a recipe can be saved successfully, with hierarchical" +
			" save/retrieval for its unique-named components",
			dependsOnMethods = "testGetByName")
	public void testSaveRecipe() {
		Long id = null;
		
		try {
			id = svc.saveOrUpdateRecipe(clientToken, rcpTest);
		} catch (Exception ex) {
			Assert.fail("Failed to save a new Recipe", ex);
		}
		
		Assert.assertNotNull(id);
		
		Recipe rcpRetr = null;
		
		try {
			rcpRetr = svc.getRecipe(clientToken, id);
		} catch (Exception ex) {
			Assert.fail("Failed to retrieve a saved Recipe", ex);
		}
		
		Assert.assertNotNull(rcpRetr);
		Assert.assertEquals(rcpRetr, rcpTest);
	}
	
	@Test(description = "Verifies that a saved recipe can be updated successfully",
			dependsOnMethods = "testSaveRecipe")
	public void testUpdateRecipe() {
		rcpTest.setSource(RecipeTestData.srcNewspaper);
		
		try {
			svc.saveOrUpdateRecipe(clientToken, rcpTest);
		} catch (Exception ex) {
			Assert.fail("Failed to update a saved Recipe", ex);
		}
		
		Recipe rcpRetr = null;
		
		try {
			rcpRetr = svc.getRecipe(clientToken, rcpTest.getId());
		} catch (Exception ex) {
			Assert.fail("Failed to retrieve a saved Recipe", ex);
		}
		
		Assert.assertNotNull(rcpRetr);
		Assert.assertEquals(rcpRetr, rcpTest);
	}
	
	@Test(description = "Verifies that a Recipe can be tagged with a new category",
			dependsOnMethods = "testUpdateRecipe")
	public void testTagRecipe() {
		Set<Recipe> rcpToTag = new HashSet<Recipe>();
		rcpToTag.add(rcpTest);
		
		tagRightTest = new RecipeCategory(RecipeTestData.catRight);
		
		try {
			svc.tagRecipes(clientToken, tagRightTest, rcpToTag);
		} catch (Exception ex) {
			Assert.fail("Failed to tag a saved Recipe", ex);
		}
		
		Recipe rcpRetr = null;
		
		try {
			rcpRetr = svc.getRecipe(clientToken, rcpTest.getId());
		} catch (Exception ex) {
			Assert.fail("Failed to retrieve a saved Recipe", ex);
		}
		
		Assert.assertNotNull(rcpRetr);
		
		boolean hasCat = false;
		for (RecipeCategoryEntry cats : rcpRetr.getCategories()) {
			if (cats.getCategory().getName().equals(RecipeTestData.catRight)) {
				hasCat = true;
				break;
			}
		}
		Assert.assertTrue(hasCat);
	}
	
	@Test(description = "Verifies that a Recipe can be untagged from a category",
			dependsOnMethods = "testTagRecipe")
	public void testUntagRecipe() {
		Set<Recipe> rcpToTag = new HashSet<Recipe>();
		rcpToTag.add(rcpTest);
		
		tagWrongTest = new RecipeCategory(RecipeTestData.catWrong);
		
		try {
			svc.untagRecipes(clientToken, tagWrongTest, rcpToTag);
		} catch (Exception ex) {
			Assert.fail("Failed to untag a saved Recipe", ex);
		}
		
		Recipe rcpRetr = null;
		
		try {
			rcpRetr = svc.getRecipe(clientToken, rcpTest.getId());
		} catch (Exception ex) {
			Assert.fail("Failed to retrieve a saved Recipe", ex);
		}
		
		Assert.assertNotNull(rcpRetr);
		
		boolean hasCat = false;
		for (RecipeCategoryEntry cats : rcpRetr.getCategories()) {
			if (cats.getCategory().getName().equals(RecipeTestData.catWrong)) {
				hasCat = true;
				break;
			}
		}
		Assert.assertFalse(hasCat);
	}
	
	@Test(description = "Verifies that Recipes can be found with respect to their Source",
			dependsOnMethods = "testUpdateRecipe")
	public void testFindBySource() {
		List<Recipe> hasRecipes = null;
		List<Recipe> noRecipes = null;
		
		try {
			hasRecipes = svc.getRecipesBySource(clientToken, RecipeTestData.srcNewspaper);
			noRecipes = svc.getRecipesBySource(clientToken, RecipeTestData.srcMagazine);
		} catch (Exception ex) {
			Assert.fail("Failed to retrieve lists of Recipes by Source", ex);
		}
		
		Assert.assertFalse(hasRecipes.isEmpty());
		Assert.assertTrue(noRecipes.isEmpty());
	}
	
	@Test(description = "Verifies that Recipes can be found with respect to their Tags",
			dependsOnMethods = "testUntagRecipe")
	public void testFindByTag() {
		List<Recipe> hasRecipes = null;
		List<Recipe> noRecipes = null;
		
		Set<RecipeCategory> tagWithRecipe = new HashSet<RecipeCategory>();
		tagWithRecipe.add(tagRightTest);
		Set<RecipeCategory> tagWithoutRecipe = new HashSet<RecipeCategory>();
		tagWithoutRecipe.add(tagWrongTest);
		
		try {
			hasRecipes = svc.getRecipesByTag(clientToken, tagWithRecipe);
			noRecipes = svc.getRecipesByTag(clientToken, tagWithoutRecipe);
		} catch (Exception ex) {
			Assert.fail("Failed to retrieve lists of Recipes by Tag", ex);
		}
		
		Assert.assertFalse(hasRecipes.isEmpty());
		Assert.assertTrue(noRecipes.isEmpty());
	}
	
	@Test(description = "Verifies that Recipes can be found with respect to their Ingredients",
			dependsOnMethods = "testUntagRecipe")
	public void testFindByIngr() {
		List<Recipe> hasRecipes = null;
		List<Recipe> noRecipes = null;
		
		Set<RecipeIngredient> ingrWithRecipe = new HashSet<RecipeIngredient>();
		ingrWithRecipe.add(ingrTest);
		Set<RecipeIngredient> ingrWithoutRecipe = new HashSet<RecipeIngredient>();
		ingrWithoutRecipe.add(new RecipeIngredient("This totally doesn't exist"));
		
		try {
			hasRecipes = svc.getRecipesByIngredient(clientToken, ingrWithRecipe);
			noRecipes = svc.getRecipesByIngredient(clientToken, ingrWithoutRecipe);
		} catch (Exception ex) {
			Assert.fail("Failed to retrieve lists of Recipes by Ingredient", ex);
		}
		
		Assert.assertFalse(hasRecipes.isEmpty());
		Assert.assertTrue(noRecipes.isEmpty());
	}
	
	@Test(dependsOnMethods = { "testFindBySource", "testFindByTag", "testFindByIngr" })
	public void testCleanupDatabase() {
		List<RecipeCategory> allTags = null;
		
		try {
			svc.cleanupDatabase(clientToken);
			allTags = svc.getAllTags(clientToken);
		} catch (Exception ex) {
			Assert.fail("Failed to clean up database", ex);
		}
		
		Assert.assertFalse(allTags.contains(tagNew));
	}
}
